#include <graphics.h>
#include <string.h>
#include <ctype.h>
#include "editline.h"

#define TIME_BLINK 9

CURSORVer::CURSORVer( EDITLineFormat *Format )
{
	DatumX = Format->StartX + 8;
	DatumY = Format->StartY;
	C_Fore = Format->C_Fore;
	C_Back = Format->C_Back;
	C_Cursor = Format->C_Cursor;
	Working = NowDisp = CurOffset = 0;
}

void CURSORVer::CursorOn( void )
{
	Working = 1;
	Display( CurOffset );
	TimeLastChg = TIME;
}

void CURSORVer::CursorOff( void )
{
	if(	Working )
	{
		Restore();
		Working = 0;
	}
}

void CURSORVer::TimeControl( void )
{
	if( Working )
	{
		DWORD t = TIME;
		if( t - TimeLastChg >= TIME_BLINK )
		{
			if( NowDisp )
				Restore();
			else
				Display( CurOffset );
			TimeLastChg = t;
		}
	}
}


void CURSORVer::GotoOffset( int OffsetX )
{
	if( Working )
	{
		if( OffsetX != CurOffset )
		{
			Restore();
			Display( OffsetX );
		}
	}
	else
		CurOffset = OffsetX;
}

// Store screen & Draw cursor at OffsetX
void CURSORVer::Display( int OffsetX )
{
	Store( OffsetX );
	setcolor( C_Cursor );
	line( SaveX, DatumY, SaveX, DatumY+19 );
	NowDisp = 1;
	CurOffset = OffsetX;
}

void CURSORVer::Store( int OffsetX )
{
	BYTE PickColor = 0;
	SaveX = DatumX + OffsetX;
	int RunY = DatumY;

	for( int CntPoints = 1 ; CntPoints <= 20 ; CntPoints++, RunY++ )
	{
		if( getpixel(SaveX,RunY) == C_Fore )
			PickColor |= 0X80;

		if( !(CntPoints%8) )
		{
			ColorPick[CntPoints/8-1] = PickColor;
			PickColor = 0;
		}
		else
			PickColor >>= 1;
	}
	PickColor >>= 3;
	ColorPick[2] = PickColor;
}

void CURSORVer::Restore( void )
{
	int RunY = DatumY;
	int PickColor;

	if( NowDisp )
	{
		for( int CntPoints = 0 ; CntPoints < 20 ; CntPoints++, RunY++ )
		{
			if( !(CntPoints%8) )
				PickColor = ColorPick[CntPoints/8];
			else
				PickColor >>= 1;

			if( PickColor & 0X01 )
				putpixel( SaveX, RunY, C_Fore );
			else
				putpixel( SaveX, RunY, C_Back );
		}
		NowDisp = 0;
	}
}


EDITLine::EDITLine( EDITLineFormat *Format, int MaxStr, BYTE *String ) :
CURSORVer( Format )
{
	StartX = Format->StartX;
	StartY = Format->StartY;
	MaxLenShow = Format->MaxLenShow;
	Width = MaxLenShow + 16;
	Heigth = 19;
	TEditX = StartX + 8;
	TEditY = StartY + 8;
	C_Highlight = Format->C_Highlight;

	StrEdit = String;
	NumChar = strlen( (char *)String );
	MaxStrSize = MaxStr;

	TabOption = 0;
	Language = 1;		// Default value : THAI Language
}

void EDITLine::Write( void )
{
	int index;

	// Copy string from StrEdit into StrShow
	CWDrv->ThaiCopyLen( StrShow, StrEdit+FirstStrShow, MaxLenShow );
	setfillstyle( SOLID_FILL, C_Back );
	CWDrv->SetColor( C_Fore );
	bar( StartX, StartY, StartX+Width, StartY+Heigth );
	CWDrv->ThaiTextXY( TEditX, TEditY, StrShow );
}

void EDITLine::HighlightText( void )
{
	CurStrPos = FirstStrShow = 0;

	CWDrv->ThaiCopyLen( StrShow, StrEdit, MaxLenShow );
	int len = CWDrv->ThaiLen( StrShow );
	setfillstyle( SOLID_FILL, C_Highlight );
	bar( StartX+4, StartY, TEditX+len+4, StartY+Heigth );
	CWDrv->SetColor( C_Back );
	CWDrv->ThaiTextXY( TEditX, TEditY, StrShow );
}

// return 1 if can delete charactor from StrEdit
// either else return 0
int EDITLine::DelChar( int Position )
{
	if( Position < NumChar && Position >= 0 )
	{
		strcpy( (char *)StrEdit+Position, (char *)StrEdit+Position+1 );
		NumChar--;
		while( CWDrv->IsNoLen(*(StrEdit+Position)) )
		{
			strcpy( (char *)StrEdit+Position, (char *)StrEdit+Position+1 );
			NumChar--;
		}
		return 1;
	}
	return 0;
}

void EDITLine::ClearEditArea( void )
{
	setfillstyle( SOLID_FILL, C_Back );
	bar( StartX, StartY, StartX+Width, StartY+Heigth );
}


// return 1 if can insert charactor into StrEdit and increase NumChar by One
// either else return 0
int EDITLine::InsertChar( unsigned char ch )
{
	int index;
	unsigned char Correct;

	if( NumChar < MaxStrSize - 1 )
	{
		if( CurStrPos )    // Is CurStrPos > 0
			Correct = CWDrv->ChkCorrect( (BYTE)*(StrEdit+CurStrPos-1), ch );
		else
			Correct = !CWDrv->IsNoLen( ch );

        if( Correct )
		{
			for( index = NumChar ; index >= CurStrPos ; index-- )
				*(StrEdit+index+1) = *(StrEdit+index);
			*(StrEdit+CurStrPos) = ch;
			NumChar++;
			return 1;
		}
	}
	return 0;
}


void EDITLine::Set( void )
{
	NumChar = strlen( (char *)StrEdit );
	if( NumChar )
	{
		ClearEditArea();
		HighlightText();
		FirstInput = 1;
		CursorOff();
	}
	else
	{
		ClearEditArea();
		CurStrPos = FirstStrShow = FirstInput = 0;
		GotoOffset( 0 );
		CursorOn();
	}
}

void EDITLine::Unset( void )
{
	if( FirstInput )
		Write();
	else
		CursorOff();
}

void EDITLine::SetString( BYTE *Str )
{
	strcpy( (char *)StrEdit, (char *)Str );
	NumChar = strlen( (char *)StrEdit );
}

// return 1 if hit Enter
// return -1 if hit Esc
// either else return 0
int EDITLine::Input( void )
{
	unsigned char WriteNew = 0;
	BYTE ch;
	KEY Key;

	Key.Key = getkeyOnly();
	if( !Key.Key )
		goto End_Module;

	if( FirstInput )
	{
		if( isprint( Key.Extend.Ascii ) )
		{
			if( Language )  // Is thai language ?
			{
				ch = Eng2Thai( &Key );
				if( CWDrv->IsNoLen( ch ) )
					goto End_Module;
			}
			else
				ch = Key.Extend.Ascii;

			CurStrPos = NumChar = 1;
			*StrEdit = ch;
			*(StrEdit+1) = '\0';
			WriteNew = 1;
			FirstInput = 0;
		}
		else if( Key.Extend.Ascii == 0X0D )
			return 1;
		else if( Key.Extend.Ascii == 0X1B )
			return -1;
		else if( Key.Extend.Ascii == 0X08 )  // Key BACKSPACE
		{
			StrEdit[--(NumChar)] = '\0';
			CurStrPos = NumChar;
			WriteNew = 1;
		}
		else
		{
			switch( Key.Extend.ScanCode ) {
			case KEY_LEFT :
			case KEY_RIGHT :
				WriteNew = 1;
				FirstInput = 0;
				break;
			case KEY_DEL :
				NumChar = 0;
				*StrEdit = '\0';
				WriteNew = 1;
				FirstInput = 0;
				break;
			case KEY_HOME :
				WriteNew = 1;
				FirstInput = 0;
				break;
			case KEY_END :
				CurStrPos = NumChar;
				WriteNew = 1;
				FirstInput = 0;
				break;
			default :
				goto End_Module;
			}   /*=========== End of Switch ==============*/
		}
	}
	else
	{
		if( isprint( Key.Extend.Ascii ) )
		{
			if( Language )       // Is thai language ?
				ch = Eng2Thai( &Key );
			else
				ch = Key.Extend.Ascii;

			if( InsertChar( ch ) )
			{
				WriteNew = 1;
				CurStrPos++;
			}
			else
				goto End_Module;
		}
		else if( Key.Extend.Ascii == 0X0D )
			return 1;
		else if( Key.Extend.Ascii == 0X1B )
			return -1;
		else if( Key.Extend.Ascii == 0X08 )  // Key BACKSPACE
		{
			if( CurStrPos > 0 )
			{
				CurStrPos--;
				if( !CWDrv->IsNoLen(*(StrEdit+CurStrPos)) )
				{
					if( FirstStrShow > 0 )
					{
						FirstStrShow--;
						while( CWDrv->IsNoLen(*(StrEdit+FirstStrShow)) )
							FirstStrShow--;
					}
				}
				DelChar( CurStrPos );
				WriteNew = 1;
			}
			else
				goto End_Module;
		}
		else
		{
			switch( Key.Extend.ScanCode ) {
			case KEY_LEFT :
				if( CurStrPos > 0  )
				{
					CurStrPos--;
					while( CWDrv->IsNoLen(*(StrEdit+CurStrPos)) )
						CurStrPos--;
					WriteNew = 1;
				}
				else
					goto End_Module;
				break;
			case KEY_RIGHT :
				if( CurStrPos < NumChar )
				{
					CurStrPos++;
					while( CWDrv->IsNoLen(*(StrEdit+CurStrPos)) )
						CurStrPos++;
					WriteNew = 1;
				}
				else
					goto End_Module;
				break;
			case KEY_DEL :
				if( DelChar(CurStrPos) )
				{
					while( CWDrv->IsNoLen(*(StrEdit+CurStrPos)) )
						DelChar( CurStrPos );
					WriteNew = 1;
				}
				else
					goto End_Module;
				break;
			case KEY_HOME :
				CurStrPos = FirstStrShow = 0;
				WriteNew = 1;
				break;
			case KEY_END :
				CurStrPos = NumChar;
				WriteNew = 1;
				break;
			default :
				goto End_Module;
			}   /*=========== End of Switch ==============*/
		}
	}

	if( WriteNew )
	{
		clrkeyOne();
		if( CurStrPos < FirstStrShow )
			FirstStrShow = CurStrPos;
		else
		{
			while( CWDrv->ThaiLen(StrEdit+FirstStrShow,
								  CurStrPos-FirstStrShow) > MaxLenShow )
			{
				FirstStrShow++;
				while( CWDrv->IsNoLen(*(StrEdit+FirstStrShow)) )
					FirstStrShow++;
			}
		}
		CursorOff();
		Write();
		GotoOffset( CWDrv->ThaiLen(StrEdit+FirstStrShow,
								   CurStrPos-FirstStrShow) );
		CursorOn();
	}

	End_Module:
	TimeControl();
	return 0;
}

